/******************************************************************************
** 
 * \file        main.c
 * \author      IOsetting | iosetting@outlook.com
 * \date        
 * \brief       Demo code of PWM in independent mode
 * \note        This will drive 3 on-board LEDs to show fade effect
 * \version     v0.1
 * \ingroup     demo
 * \remarks     test-board: HLK-W806-KIT-V1.0
 *              PWM Frequency = 40MHz / Prescaler / (Period + 1)；
                Duty Cycle(Edge Aligned)   = (Pulse + 1) / (Period + 1)
                Duty Cycle(Center Aligned) = (2 * Pulse + 1) / (2 * (Period + 1))
 *
******************************************************************************/

#include <stdio.h>
#include "wm_hal.h"
#include "gpio.h"

I2C_HandleTypeDef hi2c;
char char_buff[30];

void Error_Handler(void);
static void I2C_Init(void);

        
uint8_t i2c_send_addr(uint8_t addr)
{
    I2C->DATA = addr;           // кладем адрес слейва в регистр данных I2C
    I2C->CR_SR = I2C_CR_START | I2C_CR_WR | I2C_CR_STOP;        // выдаем на шину START, запуcкаем передачу, по окончании передачи байта выдаем STOP 
    while(I2C->CR_SR & I2C_SR_TIP){};   // ждем окончания отправки
    return (I2C->CR_SR &  I2C_SR_RXACK);        // если обнаружили NACK, возвращаем "1", если ACK - то "0"
}

int main(void)
{
   /* uint32_t div = (40*1000) / (5*100) - 1; // 40 (MHz) APB bus, calculate prescaler for I2C
        
    GPIOA->AF_S0 |= GPIO_PIN_4 | GPIO_PIN_1;      // select Alternate Function for PORTA_1 - I2C_SCL, PORTA_4 - I2C_SDA
   
    I2C->PRESCALE_L = div & 0xff;               // clock divider
    I2C->PRESCALE_H = (div >> 8) & 0xff;        // clock divider
    I2C->EN |= I2C_EN_ENABLE;                   // periph enable*/
	
	 SystemClock_Config(CPU_CLK_160M);
		
	   HAL_Init();
       GPIO_Init();
       I2C_Init();

    printf("Start I2C Address scan...\n\r");
	HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_RESET);
	HAL_Delay(1000);
    for(uint8_t addr = 0x08; addr<0xF0; addr+=2)
    {   
        if(!i2c_send_addr(addr))
        {
            sprintf(char_buff, "0x%.2X addr ACK found!\n\r", addr);
            printf(char_buff);
        }
		 HAL_GPIO_TogglePin(GPIOB, GPIO_PIN_0);
		 HAL_Delay(50);
    }
	HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);
	HAL_Delay(1000);
    printf("I2C Address scan finished \n\r");
    while (1){};
}

static void I2C_Init(void)
{
    hi2c.Instance = I2C;
    hi2c.Frequency = 400000;
    I2C_Init_Only_Write(&hi2c);

}

void Error_Handler(void)
{
   while (1){
  }
} 

void assert_failed(uint8_t *file, uint32_t line)
{
    printf("Wrong parameters value: file %s on line %d\r\n", file, line);
}